@@ -21,12 +21,13 @@ Areas for improvement:
 10. Test variety: expanding or adding functions to handle score more tests relevant for cogntive assesment, such as the MMSE.
 11. Error handling: improving error handling to better manage unexpected inputs or model failures.
 """
+
 def cerad_drawn_score(
     shape, 
     image_input,
     api_key,
     user_model="gpt-4o",
-    creativity=0,
+    creativity=None,
     reference_in_image=False,
     provide_reference=False,
     safety=False,
@@ -260,13 +261,16 @@ def cerad_drawn_score(
                 response_obj = client.chat.completions.create(
                     model=user_model,
                     messages=[{'role': 'user', 'content': prompt}],
-                    temperature=creativity
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = response_obj.choices[0].message.content
                 link1.append(reply)
             except Exception as e:
-                print("An error occurred: {e}")
-                link1.append("Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
 
         elif model_source == "Anthropic"  and valid_image:
             import anthropic
@@ -275,14 +279,17 @@ def cerad_drawn_score(
                 message = client.messages.create(
                     model=user_model,
                     max_tokens=1024,
-                    temperature=creativity,
-                    messages=[{"role": "user", "content": prompt}]
+                    messages=[{"role": "user", "content": prompt}],
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = message.content[0].text  # Anthropic returns content as list
                 link1.append(reply)
             except Exception as e:
-                print("An error occurred: {e}")
-                link1.append("Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
 
         elif model_source == "Mistral"  and valid_image:
             from mistralai import Mistral
@@ -294,14 +301,16 @@ def cerad_drawn_score(
                     messages=[
                     {'role': 'user', 'content': prompt}
                 ],
-                temperature=creativity
+                **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = response.choices[0].message.content
                 link1.append(reply)
             except Exception as e:
-                reply = None
-                print("An error occurred: {e}")
-                link1.append("Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
         #if no valid image path is provided
         elif  valid_image == False:
             reply = "invalid image path"
@@ -365,8 +374,11 @@ def cerad_drawn_score(
     normalized_data = pd.concat(normalized_data_list, ignore_index=True)
 
     categorized_data = pd.DataFrame({
-        'image_input': image_files,
-        'link1': pd.Series(link1).reset_index(drop=True),
+        'image_input': (
+            image_files.reset_index(drop=True) if isinstance(image_files, (pd.DataFrame, pd.Series)) 
+            else pd.Series(image_files)
+        ),
+        'model_response': pd.Series(link1).reset_index(drop=True),
         'json': pd.Series(extracted_jsons).reset_index(drop=True)
     })
     categorized_data = pd.concat([categorized_data, normalized_data], axis=1)
