@@ -5,7 +5,7 @@ def image_multi_class(
     categories,
     api_key,
     user_model="gpt-4o",
-    creativity=0,
+    creativity=None,
     to_csv=False,
     safety=False,
     filename="categorized_data.csv",
@@ -143,13 +143,16 @@ def image_multi_class(
                 response_obj = client.chat.completions.create(
                     model=user_model,
                     messages=[{'role': 'user', 'content': prompt}],
-                    temperature=creativity
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = response_obj.choices[0].message.content
                 link1.append(reply)
             except Exception as e:
-                print(f"An error occurred: {e}")
-                link1.append(f"Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
 
         elif model_source == "Anthropic":
             import anthropic
@@ -159,14 +162,17 @@ def image_multi_class(
                 message = client.messages.create(
                     model=user_model,
                     max_tokens=1024,
-                    temperature=creativity,
-                    messages=[{"role": "user", "content": prompt}]
+                    messages=[{"role": "user", "content": prompt}],
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = message.content[0].text
                 link1.append(reply)
             except Exception as e:
-                print(f"An error occurred: {e}")
-                link1.append(f"Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
 
         elif model_source == "Mistral":
             from mistralai import Mistral
@@ -177,13 +183,16 @@ def image_multi_class(
                     messages=[
                     {'role': 'user', 'content': prompt}
                 ],
-                temperature=creativity
+                **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = response.choices[0].message.content
                 link1.append(reply)
             except Exception as e:
-                print(f"An error occurred: {e}")
-                link1.append(f"Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
         #if no valid image path is provided
         elif  valid_image == False:
             reply = "invalid image path"
@@ -216,7 +225,7 @@ def image_multi_class(
             # Save progress so far
             temp_df = pd.DataFrame({
                 'image_input': image_files[:i+1],
-                'link1': link1,
+                'model_response': link1,
                 'json': extracted_jsons
             })
             # Normalize processed jsons so far
@@ -243,9 +252,11 @@ def image_multi_class(
         except json.JSONDecodeError:
             normalized_data_list.append(pd.DataFrame({"1": ["e"]}))
     normalized_data = pd.concat(normalized_data_list, ignore_index=True)
-
     categorized_data = pd.DataFrame({
-        'image_input': image_files,
+        'image_input': (
+            image_files.reset_index(drop=True) if isinstance(image_files, (pd.DataFrame, pd.Series)) 
+            else pd.Series(image_files)
+        ),
         'link1': pd.Series(link1).reset_index(drop=True),
         'json': pd.Series(extracted_jsons).reset_index(drop=True)
     })
@@ -266,7 +277,7 @@ def image_score_drawing(
     api_key,
     columns="numbered",
     user_model="gpt-4o-2024-11-20",
-    creativity=0,
+    creativity=None,
     to_csv=False,
     safety=False,
     filename="categorized_data.csv",
@@ -431,13 +442,16 @@ def image_score_drawing(
                 response_obj = client.chat.completions.create(
                     model=user_model,
                     messages=[{'role': 'user', 'content': prompt}],
-                    temperature=creativity
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = response_obj.choices[0].message.content
                 link1.append(reply)
             except Exception as e:
-                print(f"An error occurred: {e}")
-                link1.append(f"Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
 
         elif model_source == "Anthropic":
             import anthropic
@@ -446,14 +460,17 @@ def image_score_drawing(
                 message = client.messages.create(
                     model=user_model,
                     max_tokens=1024,
-                    temperature=creativity,
-                    messages=[{"role": "user", "content": prompt}]
+                    messages=[{"role": "user", "content": prompt}],
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = message.content[0].text  # Anthropic returns content as list
                 link1.append(reply)
             except Exception as e:
-                print(f"An error occurred: {e}")
-                link1.append(f"Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
 
         elif model_source == "Mistral":
             from mistralai import Mistral
@@ -464,13 +481,16 @@ def image_score_drawing(
                     messages=[
                     {'role': 'user', 'content': prompt}
                 ],
-                temperature=creativity
+                **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = response.choices[0].message.content
                 link1.append(reply)
             except Exception as e:
-                print(f"An error occurred: {e}")
-                link1.append(f"Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
         #if no valid image path is provided
         elif  valid_image == False:
             reply = "invalid image path"
@@ -502,7 +522,7 @@ def image_score_drawing(
             # Save progress so far
             temp_df = pd.DataFrame({
                 'image_input': image_files[:i+1],
-                'link1': link1,
+                'model_response': link1,
                 'json': extracted_jsons
             })
             # Normalize processed jsons so far
@@ -531,7 +551,10 @@ def image_score_drawing(
     normalized_data = pd.concat(normalized_data_list, ignore_index=True)
 
     categorized_data = pd.DataFrame({
-        'image_input': image_files,
+        'image_input': (
+            image_files.reset_index(drop=True) if isinstance(image_files, (pd.DataFrame, pd.Series)) 
+            else pd.Series(image_files)
+        ),
         'link1': pd.Series(link1).reset_index(drop=True),
         'json': pd.Series(extracted_jsons).reset_index(drop=True)
     })
@@ -551,7 +574,7 @@ def image_features(
     features_to_extract,
     api_key,
     user_model="gpt-4o-2024-11-20",
-    creativity=0,
+    creativity=None,
     to_csv=False,
     safety=False,
     filename="categorized_data.csv",
@@ -567,10 +590,6 @@ def image_features(
     import base64
     from pathlib import Path
 
-    if save_directory is not None and not os.path.isdir(save_directory):
-    # Directory doesn't exist - raise an exception to halt execution
-        raise FileNotFoundError(f"Directory {save_directory} doesn't exist")
-
     image_extensions = [
     '*.png', '*.jpg', '*.jpeg',
     '*.gif', '*.webp', '*.svg', '*.svgz', '*.avif', '*.apng',
@@ -595,26 +614,35 @@ def image_features(
     cat_num = len(features_to_extract)
     category_dict = {str(i+1): "0" for i in range(cat_num)}
     example_JSON = json.dumps(category_dict, indent=4)
-
-    # ensure number of categories is what user wants
-    print("\nThe image features to be extracted are:")
-    for i, cat in enumerate(features_to_extract, 1):
-        print(f"{i}. {cat}")
     
     link1 = []
     extracted_jsons = []
 
-    for i, img_path in enumerate(
-        tqdm(image_files, desc="Categorising images"), start=0):
+    for i, img_path in enumerate(tqdm(image_files, desc="Scoring images"), start=0):
+    # Check validity first
         if img_path is None or not os.path.exists(img_path):
             link1.append("Skipped NaN input or invalid path")
             extracted_jsons.append("""{"no_valid_image": 1}""")
             continue  # Skip the rest of the loop iteration
-    # encode this specific image once
-        with open(img_path, "rb") as f:
-            encoded = base64.b64encode(f.read()).decode("utf-8")
-        ext = Path(img_path).suffix.lstrip(".").lower()
-        encoded_image = f"data:image/{ext};base64,{encoded}"
+        
+    # Only open the file if path is valid
+        if os.path.isdir(img_path):
+            encoded = "Not a Valid Image, contains file path"
+        else:
+            try:
+                with open(img_path, "rb") as f:
+                    encoded = base64.b64encode(f.read()).decode("utf-8")
+            except Exception as e:
+                    encoded = f"Error: {str(e)}"
+    # Handle extension safely
+        if encoded.startswith("Error:") or encoded == "Not a Valid Image, contains file path":
+            encoded_image = encoded
+            valid_image = False
+            
+        else:
+            ext = Path(img_path).suffix.lstrip(".").lower()
+            encoded_image = f"data:image/{ext};base64,{encoded}"
+            valid_image = True
 
         if model_source == "OpenAI" or model_source == "Mistral":
             prompt = [
@@ -687,13 +715,16 @@ def image_features(
                 response_obj = client.chat.completions.create(
                     model=user_model,
                     messages=[{'role': 'user', 'content': prompt}],
-                    temperature=creativity
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = response_obj.choices[0].message.content
                 link1.append(reply)
             except Exception as e:
-                print(f"An error occurred: {e}")
-                link1.append(f"Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
 
         elif model_source == "Perplexity":
             from openai import OpenAI
@@ -702,13 +733,17 @@ def image_features(
                 response_obj = client.chat.completions.create(
                     model=user_model,
                     messages=[{'role': 'user', 'content': prompt}],
-                    temperature=creativity
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = response_obj.choices[0].message.content
                 link1.append(reply)
             except Exception as e:
-                print(f"An error occurred: {e}")
-                link1.append(f"Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
+
         elif model_source == "Anthropic":
             import anthropic
             client = anthropic.Anthropic(api_key=api_key)
@@ -716,14 +751,18 @@ def image_features(
                 message = client.messages.create(
                     model=user_model,
                     max_tokens=1024,
-                    temperature=creativity,
-                    messages=[{"role": "user", "content": prompt}]
+                    messages=[{"role": "user", "content": prompt}],
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = message.content[0].text  # Anthropic returns content as list
                 link1.append(reply)
             except Exception as e:
-                print(f"An error occurred: {e}")
-                link1.append(f"Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
+
         elif model_source == "Mistral":
             from mistralai import Mistral
             client = Mistral(api_key=api_key)
@@ -733,13 +772,17 @@ def image_features(
                     messages=[
                     {'role': 'user', 'content': prompt}
                 ],
-                temperature=creativity
+                **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = response.choices[0].message.content
                 link1.append(reply)
             except Exception as e:
-                print(f"An error occurred: {e}")
-                link1.append(f"Error processing input: {e}")
+                if "model" in str(e).lower():
+                    raise ValueError(f"Invalid OpenAI model '{user_model}': {e}")
+                else:
+                    print("An error occurred: {e}")
+                    link1.append("Error processing input: {e}")
+
         elif  valid_image == False:
             print("Skipped NaN input or invalid path")
             reply = None
@@ -797,8 +840,11 @@ def image_features(
     normalized_data = pd.concat(normalized_data_list, ignore_index=True)
 
     categorized_data = pd.DataFrame({
-        'image_input': image_files,
-        'link1': pd.Series(link1).reset_index(drop=True),
+        'image_input': (
+            image_files.reset_index(drop=True) if isinstance(image_files, (pd.DataFrame, pd.Series)) 
+            else pd.Series(image_files)
+        ),
+        'model_response': pd.Series(link1).reset_index(drop=True),
         'json': pd.Series(extracted_jsons).reset_index(drop=True)
     })
     categorized_data = pd.concat([categorized_data, normalized_data], axis=1)
