@@ -8,7 +8,7 @@ def explore_corpus(
     cat_num=10,
     divisions=5,
     user_model="gpt-4o-2024-11-20",
-    creativity=0,
+    creativity=None,
     filename="corpus_exploration.csv",
     model_source="OpenAI"
 ):
@@ -57,7 +57,7 @@ Number your categories from 1 through {cat_num} and be concise with the category
                          The research question is: {research_question}""" if research_question else "You are a helpful assistant."},
                         {'role': 'user', 'content': prompt}
                     ],
-                    temperature=creativity
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = response_obj.choices[0].message.content
                 responses.append(reply)
@@ -107,7 +107,7 @@ def explore_common_categories(
     cat_num=10,
     divisions=5,
     user_model="gpt-4o",
-    creativity=0,
+    creativity=None,
     specificity="broad",
     research_question=None,
     filename=None,
@@ -158,7 +158,7 @@ Number your categories from 1 through {cat_num} and be concise with the category
                          The research question is: {research_question}""" if research_question else "You are a helpful assistant."},
                         {'role': 'user', 'content': prompt}
                     ],
-                    temperature=creativity
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                 reply = response_obj.choices[0].message.content
                 responses.append(reply)
@@ -225,8 +225,9 @@ def multi_class(
     categories,
     api_key,
     user_model="gpt-4o",
-    creativity=0,
+    creativity=None,
     safety=False,
+    to_csv=False,
     filename="categorized_data.csv",
     save_directory=None,
     model_source="OpenAI"
@@ -271,7 +272,7 @@ Provide your work in JSON format where the number belonging to each category is
                     response_obj = client.chat.completions.create(
                     model=user_model,
                     messages=[{'role': 'user', 'content': prompt}],
-                    temperature=creativity
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                     reply = response_obj.choices[0].message.content
                     link1.append(reply)
@@ -285,7 +286,7 @@ Provide your work in JSON format where the number belonging to each category is
                     response_obj = client.chat.completions.create(
                         model=user_model,
                         messages=[{'role': 'user', 'content': prompt}],
-                        temperature=creativity
+                        **({"temperature": creativity} if creativity is not None else {})
                     )
                     reply = response_obj.choices[0].message.content
                     link1.append(reply)
@@ -299,7 +300,7 @@ Provide your work in JSON format where the number belonging to each category is
                     message = client.messages.create(
                     model=user_model,
                     max_tokens=1024,
-                    temperature=creativity,
+                    **({"temperature": creativity} if creativity is not None else {})
                     messages=[{"role": "user", "content": prompt}]
                 )
                     reply = message.content[0].text  # Anthropic returns content as list
@@ -307,6 +308,37 @@ Provide your work in JSON format where the number belonging to each category is
                 except Exception as e:
                     print(f"An error occurred: {e}")
                     link1.append(f"Error processing input: {e}")
+                    
+            elif model_source == "Google":
+                import requests
+                url = f"https://generativelanguage.googleapis.com/v1beta/models/{user_model}:generateContent"
+                try:
+                    headers = {
+                        "x-goog-api-key": api_key,
+                        "Content-Type": "application/json"
+                        }
+                    
+                    payload = {
+                        "contents": [{
+                            "parts": [{"text": prompt}]
+                            }],
+                            **({"generationConfig": {"temperature": creativity}} if creativity is not None else {})
+                            }
+                    
+                    response = requests.post(url, headers=headers, json=payload)
+                    response.raise_for_status()  # Raise exception for HTTP errors
+                    result = response.json()
+
+                    if "candidates" in result and result["candidates"]:
+                        reply = result["candidates"][0]["content"]["parts"][0]["text"]
+                    else:
+                        reply = "No response generated"
+
+                    link1.append(reply)
+                except Exception as e:
+                    print(f"An error occurred: {e}")
+                    link1.append(f"Error processing input: {e}")
+
             elif model_source == "Mistral":
                 from mistralai import Mistral
                 client = Mistral(api_key=api_key)
@@ -316,7 +348,7 @@ Provide your work in JSON format where the number belonging to each category is
                     messages=[
                         {'role': 'user', 'content': prompt}
                     ],
-                    temperature=creativity
+                    **({"temperature": creativity} if creativity is not None else {})
                 )
                     reply = response.choices[0].message.content
                     link1.append(reply)
@@ -359,7 +391,7 @@ Provide your work in JSON format where the number belonging to each category is
                     normalized_data_list.append(pd.DataFrame({"1": ["e"]}))
             normalized_data = pd.concat(normalized_data_list, ignore_index=True)
             temp_df = pd.concat([temp_df, normalized_data], axis=1)
-            # Save to CSV
+            # save to CSV
             if save_directory is None:
                 save_directory = os.getcwd()
             temp_df.to_csv(os.path.join(save_directory, filename), index=False)
@@ -373,16 +405,15 @@ Provide your work in JSON format where the number belonging to each category is
         except json.JSONDecodeError:
             normalized_data_list.append(pd.DataFrame({"1": ["e"]}))
     normalized_data = pd.concat(normalized_data_list, ignore_index=True)
-
     categorized_data = pd.DataFrame({
-        'survey_response': survey_input.reset_index(drop=True),
-        'link1': pd.Series(link1).reset_index(drop=True),
+        'survey_input': (
+            survey_input.reset_index(drop=True) if isinstance(survey_input, (pd.DataFrame, pd.Series)) 
+            else pd.Series(survey_input)
+        ),
+        'model_response': pd.Series(link1).reset_index(drop=True),
         'json': pd.Series(extracted_jsons).reset_index(drop=True)
     })
     categorized_data = pd.concat([categorized_data, normalized_data], axis=1)
-    
-    if columns != "numbered": #if user wants text columns
-        categorized_data.columns = list(categorized_data.columns[:3]) + categories[:len(categorized_data.columns) - 3]
 
     if to_csv:
         if save_directory is None:
